#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.watch = exports.build = exports.DEFAULT_OUTPUT_DIR_PATH = void 0;
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const chokidar_1 = __importDefault(require("chokidar"));
const mkdirp_1 = __importDefault(require("mkdirp"));
const slash_1 = __importDefault(require("slash"));
const ejs_1 = __importDefault(require("ejs"));
const template_1 = require("./template");
exports.DEFAULT_OUTPUT_DIR_PATH = './node_modules';
async function buildHelpers(config) {
    const routesInfo = {};
    const routeIds = [];
    const handleRoutesRecursive = (parentId, parentPath = []) => {
        let routes = Object.values(config.routes).filter((route) => route.parentId === parentId);
        routes.forEach((route) => {
            let currentPath = parentPath;
            routeIds.push(route.id);
            if (route.id === 'root') {
                routesInfo['/'] = {
                    fileName: route.file,
                    params: [],
                };
            }
            else {
                currentPath = [...currentPath, route];
                const fullPath = dedupPrefixSlash(currentPath.reduce((acc, curr) => [acc, trimSlash(curr.path)].filter(p => p != undefined).join('/'), ''));
                const paramsNames = parse(currentPath);
                // Expand out the paths into all of the possible variants, taking into
                // account optional segments that aren't params/dynamic.
                for (const pathVariant of expandOptionalStaticSegments(fullPath)) {
                    routesInfo[pathVariant] = {
                        fileName: route.file,
                        params: paramsNames
                    };
                }
            }
            handleRoutesRecursive(route.id, currentPath);
        });
    };
    handleRoutesRecursive();
    return [routesInfo, routeIds];
}
function expandOptionalStaticSegments(path) {
    // Split the path at the point where `/` is the next char (positive lookahead regex)
    const segments = path.split(/(?=\/)/g);
    // This turns '/foo/bar' -> ['/foo', '/bar']
    let paths = [''];
    for (const e of segments) {
        if (!e.endsWith('?') || e.startsWith('/:')) {
            // If a given segment is not optional or dynamic, then we just append it to
            // each path variant we have so far: ['/foo'] => ['/foo/bar']
            paths = paths.map((p) => p + e);
        }
        else {
            // If a given segment is optional, we append a copy of the existing paths
            // with the optional segment appended:
            // ['/foo'] => ['/foo', '/foo/bar']
            paths.push(...paths.map((p) => p + e.slice(0, -1)));
        }
    }
    return paths;
}
async function build(remixRoot, remixConfig, options) {
    const [routesInfo, routeIds] = await buildHelpers(remixConfig);
    generate(remixRoot, remixConfig, routesInfo, routeIds, options);
}
exports.build = build;
async function watch(remixRoot, remixConfig, options) {
    build(remixRoot, remixConfig, options);
    const { rootDirectory, appDirectory } = remixConfig;
    chokidar_1.default
        .watch([
        path.join(appDirectory, 'routes/**/*.{ts,tsx}'),
        path.join(rootDirectory, 'remix.config.js'),
    ])
        .on('change', () => {
        build(remixRoot, remixConfig, options);
    });
    console.log('Watching for routes changes...');
}
exports.watch = watch;
function generate(remixRoot, remixConfig, routesInfo, routeIds, options) {
    const outputPath = path.join(remixRoot, options.outputDirPath);
    const relativeAppDirPath = (0, slash_1.default)(path.relative(outputPath, remixConfig.appDirectory));
    routeIds.sort((a, b) => a.localeCompare(b));
    const tsCode = ejs_1.default.render(template_1.template, {
        strictMode: options.strict,
        relativeAppDirPath,
        routes: Object.entries(routesInfo).map(([route, { fileName, params }]) => ({
            route,
            params,
            fileName: (0, slash_1.default)(fileName.replace(/\.tsx?$/, '')),
        })).sort((a, b) => a.route.localeCompare(b.route)),
        routeIds,
    });
    if (!fs.existsSync(outputPath)) {
        mkdirp_1.default.sync(outputPath);
    }
    fs.writeFileSync(path.join(outputPath, 'remix-routes.d.ts'), tsCode);
}
function parse(routes) {
    const paramNames = [];
    routes.forEach((route) => {
        return (route.path &&
            paramNames.push(...route.path
                .split('/')
                .filter((seg) => seg.startsWith(':') || seg == '*')
                .map((param) => param.replace(':', '').replace('*', '"*"'))));
    });
    return paramNames;
}
function dedupPrefixSlash(path) {
    return path.replace(/^\/+/, '/');
}
function trimSlash(path) {
    if (!path)
        return path;
    return path.replace(/\/+$/, '');
}
